use strict;
use warnings;
use lib 't/lib';
use Test::More qw(no_plan);
use My::Schema;

my $schema = My::Schema->connect('dbi:SQLite:dbname=:memory:');

$schema->deploy;

my $people = $schema->resultset('Person');

my $bob = $people->create({
    name => 'Bob Testuser',
});

$bob->create_related(assigned_tasks => {
    summary => 'Task A',
    urgency => 10,
});
$bob->create_related(assigned_tasks => {
    summary => 'Task B',
    urgency => 20,
});
$bob->create_related(assigned_tasks => {
    summary => 'Task C',
    urgency => 30,
});

my $test_rs = $schema
    ->resultset('Person')
    ->search({ 'me.name' => { -like => 'Bob%' } });

is $test_rs->with_parameterized_join(
    urgent_assigned_tasks => { urgency_threshold => 20 },
)->count, 1, 'joined';

is $test_rs->with_parameterized_join(
    urgent_assigned_tasks => { urgency_threshold => 200 },
)->count, 1, 'not joined';
